@@ -20,26 +20,28 @@
 #endif
 
 #include <getopt.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
 #include <errno.h>
 
+#include "closeout.h"
 #include "error.h"
 #include "progname.h"
 #include "relocatable.h"
 #include "basename.h"
-#include "xmalloc.h"
+#include "xalloc.h"
 #include "exit.h"
 
 #include "gettext.h"
 
 #define _(str) gettext (str)
 
-/* If nonzero expand escape sequences in strings before looking in the
+/* If true, expand escape sequences in strings before looking in the
    message catalog.  */
-int do_expand;
+static int do_expand;
 
 /* Long options.  */
 static const struct option long_options[] =
@@ -68,11 +70,11 @@ main (int argc, char *argv[])
   unsigned long n;
 
   /* Default values for command line options.  */
-  int do_help = 0;
-  int do_version = 0;
+  bool do_help = false;
+  bool do_version = false;
   const char *domain = getenv ("TEXTDOMAIN");
   const char *domaindir = getenv ("TEXTDOMAINDIR");
-  do_expand = 0;
+  do_expand = false;
 
   /* Set program name for message texts.  */
   set_program_name (argv[0]);
@@ -86,6 +88,9 @@ main (int argc, char *argv[])
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
   textdomain (PACKAGE);
 
+  /* Ensure that write errors on stdout are detected.  */
+  atexit (close_stdout);
+
   /* Parse command line options.  */
   while ((optchar = getopt_long (argc, argv, "+d:eEhV", long_options, NULL))
 	 != EOF)
@@ -97,16 +102,16 @@ main (int argc, char *argv[])
       domain = optarg;
       break;
     case 'e':
-      do_expand = 1;
+      do_expand = true;
       break;
     case 'E':
       /* Ignore.  Just for compatibility.  */
       break;
     case 'h':
-      do_help = 1;
+      do_help = true;
       break;
     case 'V':
-      do_version = 1;
+      do_version = true;
       break;
     default:
       usage (EXIT_FAILURE);
