@@ -27,11 +27,11 @@
 
 #include "format.h"
 #include "c-ctype.h"
-#include "xmalloc.h"
+#include "xalloc.h"
 #include "xerror.h"
 #include "format-invalid.h"
 #include "error.h"
-#include "progname.h"
+#include "error-progname.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
