@@ -27,10 +27,12 @@
 #include <stdlib.h>
 #include <locale.h>
 
+#include "closeout.h"
 #include "dir-list.h"
 #include "str-list.h"
 #include "file-list.h"
 #include "error.h"
+#include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
 #include "basename.h"
@@ -69,6 +71,8 @@ static const struct option long_options[] =
   { "sort-by-file", no_argument, NULL, 'F' },
   { "sort-output", no_argument, NULL, 's' },
   { "strict", no_argument, NULL, 'S' },
+  { "stringtable-input", no_argument, NULL, CHAR_MAX + 3 },
+  { "stringtable-output", no_argument, NULL, CHAR_MAX + 4 },
   { "to-code", required_argument, NULL, 't' },
   { "unique", no_argument, NULL, 'u' },
   { "use-first", no_argument, NULL, CHAR_MAX + 1 },
@@ -116,6 +120,9 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
   textdomain (PACKAGE);
 
+  /* Ensure that write errors on stdout are detected.  */
+  atexit (close_stdout);
+
   /* Set default values for variables.  */
   do_help = false;
   do_version = false;
@@ -170,7 +177,7 @@ main (int argc, char **argv)
 
       case 'F':
 	sort_by_filepos = true;
-        break;
+	break;
 
       case 'h':
 	do_help = true;
@@ -210,8 +217,8 @@ main (int argc, char **argv)
 	break;
 
       case 'u':
-        less_than = 2;
-        break;
+	less_than = 2;
+	break;
 
       case 'V':
 	do_version = true;
@@ -235,6 +242,14 @@ main (int argc, char **argv)
 	message_page_width_ignore ();
 	break;
 
+      case CHAR_MAX + 3: /* --stringtable-input */
+	input_syntax = syntax_stringtable;
+	break;
+
+      case CHAR_MAX + 4: /* --stringtable-output */
+	message_print_syntax_stringtable ();
+	break;
+
       default:
 	usage (EXIT_FAILURE);
 	/* NOTREACHED */
@@ -270,8 +285,8 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
   /* Check the message selection criteria for sanity.  */
   if (more_than >= less_than || less_than < 2)
     error (EXIT_FAILURE, 0,
-           _("impossible selection criteria specified (%d < n < %d)"),
-           more_than, less_than);
+	   _("impossible selection criteria specified (%d < n < %d)"),
+	   more_than, less_than);
 
   /* Determine list of files we have to process.  */
   if (files_from != NULL)
@@ -285,6 +300,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
   /* Read input files, then filter, convert and merge messages.  */
   result = catenate_msgdomain_list (file_list,
 				    output_syntax != syntax_properties
+				    && output_syntax != syntax_stringtable
 				    ? to_code
 				    : "UTF-8");
 
@@ -367,6 +383,9 @@ Message selection:\n"));
 Input file syntax:\n"));
       printf (_("\
   -P, --properties-input      input files are in Java .properties syntax\n"));
+      printf (_("\
+      --stringtable-input     input files are in NeXTstep/GNUstep .strings\n\
+                              syntax\n"));
       printf ("\n");
       printf (_("\
 Output details:\n"));
@@ -392,6 +411,8 @@ Output details:\n"));
       printf (_("\
   -p, --properties-output     write out a Java .properties file\n"));
       printf (_("\
+      --stringtable-output    write out a NeXTstep/GNUstep .strings file\n"));
+      printf (_("\
   -w, --width=NUMBER          set output page width\n"));
       printf (_("\
       --no-wrap               do not break long message lines, longer than\n\
