@@ -32,12 +32,16 @@
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
+#elif defined _MSC_VER || defined __MINGW32__
+# include <io.h>
 #endif
 
 #include <fnmatch.h>
 
+#include "closeout.h"
 #include "dir-list.h"
 #include "error.h"
+#include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
 #include "basename.h"
@@ -46,7 +50,7 @@
 #include "write-po.h"
 #include "str-list.h"
 #include "msgl-charset.h"
-#include "xmalloc.h"
+#include "xalloc.h"
 #include "exit.h"
 #include "full-write.h"
 #include "findprog.h"
@@ -103,6 +107,8 @@ static const struct option long_options[] =
   { "sort-by-file", no_argument, NULL, CHAR_MAX + 4 },
   { "sort-output", no_argument, NULL, CHAR_MAX + 5 },
   { "strict", no_argument, NULL, 'S' },
+  { "stringtable-input", no_argument, NULL, CHAR_MAX + 7 },
+  { "stringtable-output", no_argument, NULL, CHAR_MAX + 8 },
   { "version", no_argument, NULL, 'V' },
   { "width", required_argument, NULL, 'w' },
   { NULL, 0, NULL, 0 }
@@ -149,6 +155,9 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
   textdomain (PACKAGE);
 
+  /* Ensure that write errors on stdout are detected.  */
+  atexit (close_stdout);
+
   /* Set default values for variables.  */
   do_help = false;
   do_version = false;
@@ -283,6 +292,14 @@ main (int argc, char **argv)
 	message_page_width_ignore ();
 	break;
 
+      case CHAR_MAX + 7: /* --stringtable-input */
+	input_syntax = syntax_stringtable;
+	break;
+
+      case CHAR_MAX + 8: /* --stringtable-output */
+	message_print_syntax_stringtable ();
+	break;
+
       default:
 	usage (EXIT_FAILURE);
 	break;
@@ -481,6 +498,8 @@ expressions if -E is given, or fixed strings if -F is given.\n\
 Input file syntax:\n"));
       printf (_("\
   -P, --properties-input      input file is in Java .properties syntax\n"));
+      printf (_("\
+      --stringtable-input     input file is in NeXTstep/GNUstep .strings syntax\n"));
       printf ("\n");
       printf (_("\
 Output details:\n"));
@@ -501,6 +520,8 @@ Output details:\n"));
       printf (_("\
   -p, --properties-output     write out a Java .properties file\n"));
       printf (_("\
+      --stringtable-output    write out a NeXTstep/GNUstep .strings file\n"));
+      printf (_("\
   -w, --width=NUMBER          set output page width\n"));
       printf (_("\
       --no-wrap               do not break long message lines, longer than\n\
@@ -574,7 +595,7 @@ is_string_selected (int grep_pass, const char *str, size_t len)
 
       /* Open a pipe to a grep subprocess.  */
       child = create_pipe_out ("grep", grep_path, grep_argv[grep_pass],
-			       DEV_NULL, false, true, fd);
+			       DEV_NULL, false, true, true, fd);
 
       if (full_write (fd[0], str, len) < len)
 	error (EXIT_FAILURE, errno,
@@ -583,7 +604,7 @@ is_string_selected (int grep_pass, const char *str, size_t len)
       close (fd[0]);
 
       /* Remove zombie process from process list, and retrieve exit status.  */
-      exitstatus = wait_subprocess (child, "grep", true);
+      exitstatus = wait_subprocess (child, "grep", false, true, true);
       return (exitstatus == 0);
     }
   else
