@@ -27,9 +27,11 @@
 #include <stdlib.h>
 #include <locale.h>
 
+#include "closeout.h"
 #include "dir-list.h"
 #include "str-list.h"
 #include "error.h"
+#include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
 #include "basename.h"
@@ -68,6 +70,8 @@ static const struct option long_options[] =
   { "sort-by-file", no_argument, NULL, 'F' },
   { "sort-output", no_argument, NULL, 's' },
   { "strict", no_argument, NULL, 'S' },
+  { "stringtable-input", no_argument, NULL, CHAR_MAX + 3 },
+  { "stringtable-output", no_argument, NULL, CHAR_MAX + 4 },
   { "to-code", required_argument, NULL, 't' },
   { "unique", no_argument, NULL, 'u' },
   { "use-first", no_argument, NULL, CHAR_MAX + 1 },
@@ -111,6 +115,9 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
   textdomain (PACKAGE);
 
+  /* Ensure that write errors on stdout are detected.  */
+  atexit (close_stdout);
+
   /* Set default values for variables.  */
   do_help = false;
   do_version = false;
@@ -128,7 +135,7 @@ main (int argc, char **argv)
 	break;
 
       case 'd':
-        more_than = 1;
+	more_than = 1;
 	less_than = INT_MAX;
 	break;
 
@@ -146,7 +153,7 @@ main (int argc, char **argv)
 
       case 'F':
 	sort_by_filepos = true;
-        break;
+	break;
 
       case 'h':
 	do_help = true;
@@ -185,9 +192,9 @@ main (int argc, char **argv)
 	break;
 
       case 'u':
-        more_than = 0;
+	more_than = 0;
 	less_than = 2;
-        break;
+	break;
 
       case 'V':
 	do_version = true;
@@ -211,6 +218,14 @@ main (int argc, char **argv)
 	message_page_width_ignore ();
 	break;
 
+      case CHAR_MAX + 3: /* --stringtable-input */
+	input_syntax = syntax_stringtable;
+	break;
+
+      case CHAR_MAX + 4: /* --stringtable-output */
+	message_print_syntax_stringtable ();
+	break;
+
       default:
 	usage (EXIT_FAILURE);
 	/* NOTREACHED */
@@ -333,6 +348,8 @@ Message selection:\n"));
 Input file syntax:\n"));
       printf (_("\
   -P, --properties-input      input file is in Java .properties syntax\n"));
+      printf (_("\
+      --stringtable-input     input file is in NeXTstep/GNUstep .strings syntax\n"));
       printf ("\n");
       printf (_("\
 Output details:\n"));
@@ -358,6 +375,8 @@ Output details:\n"));
       printf (_("\
   -p, --properties-output     write out a Java .properties file\n"));
       printf (_("\
+      --stringtable-output    write out a NeXTstep/GNUstep .strings file\n"));
+      printf (_("\
   -w, --width=NUMBER          set output page width\n"));
       printf (_("\
       --no-wrap               do not break long message lines, longer than\n\
