@@ -33,7 +33,9 @@
 # include <unistd.h>
 #endif
 
+#include "closeout.h"
 #include "error.h"
+#include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
 #include "basename.h"
@@ -100,6 +102,9 @@ main (int argc, char *argv[])
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
   textdomain (PACKAGE);
 
+  /* Ensure that write errors on stdout are detected.  */
+  atexit (close_stdout);
+
   /* Set default values for variables.  */
   do_help = false;
   do_version = false;
@@ -130,7 +135,7 @@ main (int argc, char *argv[])
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 "),
-              "2001-2003");
+	      "2001-2003");
       printf (_("Written by %s.\n"), "Bruno Haible");
       exit (EXIT_SUCCESS);
     }
@@ -224,7 +229,8 @@ execute_it (const char *progname,
 {
   (void) private_data;
 
-  return execute (progname, prog_path, prog_argv, true, false, false, false)
+  return execute (progname, prog_path, prog_argv, true, false, false, true,
+		  false)
 	 != 0;
 }
 
@@ -281,7 +287,8 @@ fetch (const char *url, const char *file)
 	argv[0] = "wget";
 	argv[1] = "--version";
 	argv[2] = NULL;
-	exitstatus = execute ("wget", "wget", argv, false, true, true, false);
+	exitstatus = execute ("wget", "wget", argv, false, true, true, true,
+			      false);
 	wget_present = (exitstatus == 0);
 	wget_tested = true;
       }
@@ -297,7 +304,8 @@ fetch (const char *url, const char *file)
 	argv[4] = "-T"; argv[5] = "30";
 	argv[6] = (char *) url;
 	argv[7] = NULL;
-	exitstatus = execute ("wget", "wget", argv, false, false, false, false);
+	exitstatus = execute ("wget", "wget", argv, false, false, false, true,
+			      false);
 	if (exitstatus != 127)
 	  {
 	    if (exitstatus != 0)
@@ -322,7 +330,8 @@ fetch (const char *url, const char *file)
 	argv[0] = "lynx";
 	argv[1] = "--version";
 	argv[2] = NULL;
-	exitstatus = execute ("lynx", "lynx", argv, false, true, true, false);
+	exitstatus = execute ("lynx", "lynx", argv, false, true, true, true,
+			      false);
 	lynx_present = (exitstatus == 0);
 	lynx_tested = true;
       }
@@ -336,7 +345,8 @@ fetch (const char *url, const char *file)
 	argv[1] = "-source";
 	argv[2] = (char *) url;
 	argv[3] = NULL;
-	exitstatus = execute ("lynx", "lynx", argv, false, false, false, false);
+	exitstatus = execute ("lynx", "lynx", argv, false, false, false, true,
+			      false);
 	if (exitstatus != 127)
 	  {
 	    if (exitstatus != 0)
@@ -361,7 +371,8 @@ fetch (const char *url, const char *file)
 	argv[0] = "curl";
 	argv[1] = "--version";
 	argv[2] = NULL;
-	exitstatus = execute ("curl", "curl", argv, false, true, true, false);
+	exitstatus = execute ("curl", "curl", argv, false, true, true, true,
+			      false);
 	curl_present = (exitstatus == 0 || exitstatus == 2);
 	curl_tested = true;
       }
@@ -375,7 +386,8 @@ fetch (const char *url, const char *file)
 	argv[1] = "--silent";
 	argv[2] = (char *) url;
 	argv[3] = NULL;
-	exitstatus = execute ("curl", "curl", argv, false, false, false, false);
+	exitstatus = execute ("curl", "curl", argv, false, false, false, true,
+			      false);
 	if (exitstatus != 127)
 	  {
 	    if (exitstatus != 0)
