@@ -40,10 +40,12 @@
 #include "linebreak.h"
 #include "msgl-ascii.h"
 #include "write-properties.h"
-#include "xmalloc.h"
+#include "write-stringtable.h"
+#include "xalloc.h"
 #include "strstr.h"
+#include "fwriteerror.h"
 #include "exit.h"
-#include "progname.h"
+#include "error-progname.h"
 #include "error.h"
 #include "xerror.h"
 #include "gettext.h"
@@ -51,7 +53,7 @@
 /* Our regular abbreviation.  */
 #define _(str) gettext (str)
 
-#ifdef HAVE_PUTC_UNLOCKED
+#if HAVE_DECL_PUTC_UNLOCKED
 # undef putc
 # define putc putc_unlocked
 #endif
@@ -63,7 +65,7 @@
 /* Convert IS_FORMAT in the context of programming language LANG to a flag
    string for use in #, flags.  */
 
-static const char *
+const char *
 make_format_description_string (enum is_format is_format, const char *lang,
 				bool debug)
 {
@@ -78,6 +80,7 @@ make_format_description_string (enum is_format is_format, const char *lang,
 	  break;
 	}
       /* FALLTHROUGH */
+    case yes_according_to_context:
     case yes:
       sprintf (result, " %s-format", lang);
       break;
@@ -95,7 +98,7 @@ make_format_description_string (enum is_format is_format, const char *lang,
 
 /* Return true if IS_FORMAT is worth mentioning in a #, flags list.  */
 
-static bool
+bool
 significant_format_p (enum is_format is_format)
 {
   return is_format != undecided && is_format != impossible;
@@ -385,6 +388,16 @@ message_print_syntax_properties ()
 }
 
 
+/* Whether to output a file in NeXTstep/GNUstep .strings syntax.  */
+static bool use_syntax_stringtable = false;
+
+void
+message_print_syntax_stringtable ()
+{
+  use_syntax_stringtable = true;
+}
+
+
 /* ================ msgdomain_list_print() and subroutines. ================ */
 
 
@@ -399,7 +412,7 @@ memcpy_small (void *dst, const void *src, size_t n)
 
       *q = *p;
       if (--n > 0)
-        do *++q = *++p; while (--n > 0);
+	do *++q = *++p; while (--n > 0);
     }
 }
 
@@ -474,9 +487,9 @@ wrap (FILE *fp, const char *line_prefix, const char *name, const char *value,
   do
     {
       /* The \a and \v escapes were added by the ANSI C Standard.
-         Prior to the Standard, most compilers did not have them.
-         Because we need the same program on all platforms we don't provide
-         support for them here.  Thus we only support \b\f\n\r\t.  */
+	 Prior to the Standard, most compilers did not have them.
+	 Because we need the same program on all platforms we don't provide
+	 support for them here.  Thus we only support \b\f\n\r\t.  */
 #     define is_escape(c) \
        ((c) == '\b' || (c) == '\f' || (c) == '\n' || (c) == '\r' || (c) == '\t')
 
@@ -746,7 +759,7 @@ internationalized messages should not contain the `\\%c' escape sequence"),
 	 this is the first line, and if the indented style is being
 	 used.  INDENT-F.  */
       if (line_prefix != NULL)
-        fputs (line_prefix, fp);
+	fputs (line_prefix, fp);
       if (first_line)
 	{
 	  fputs (name, fp);
@@ -1036,10 +1049,15 @@ msgdomain_list_print (msgdomain_list_ty *mdlp, const char *filename,
     }
 
   /* Check whether the output format can accomodate all messages.  */
-  if (use_syntax_properties)
+  if (use_syntax_properties || use_syntax_stringtable)
     {
       if (mdlp->nitems > 1)
-	error (EXIT_FAILURE, 0, _("Cannot output multiple translation domains into a single file with Java .properties syntax. Try using PO file syntax instead."));
+	{
+	  if (use_syntax_properties)
+	    error (EXIT_FAILURE, 0, _("Cannot output multiple translation domains into a single file with Java .properties syntax. Try using PO file syntax instead."));
+	  if (use_syntax_stringtable)
+	    error (EXIT_FAILURE, 0, _("Cannot output multiple translation domains into a single file with NeXTstep/GNUstep .strings syntax."));
+	}
       if (mdlp->nitems == 1)
 	{
 	  message_list_ty *mlp = mdlp->item[0]->messages;
@@ -1061,9 +1079,14 @@ msgdomain_list_print (msgdomain_list_ty *mdlp, const char *filename,
 	  if (has_plural != NULL)
 	    {
 	      error_with_progname = false;
-	      error_at_line (EXIT_FAILURE, 0,
-			     has_plural->file_name, has_plural->line_number,
-			     _("message catalog has plural form translations, but the output format does not support them. Try generating a Java class using \"msgfmt --java\", instead of a properties file."));
+	      if (use_syntax_properties)
+		error_at_line (EXIT_FAILURE, 0,
+			       has_plural->file_name, has_plural->line_number,
+			       _("message catalog has plural form translations, but the output format does not support them. Try generating a Java class using \"msgfmt --java\", instead of a properties file."));
+	      if (use_syntax_stringtable)
+		error_at_line (EXIT_FAILURE, 0,
+			       has_plural->file_name, has_plural->line_number,
+			       _("message catalog has plural form translations, but the output format does not support them."));
 	      error_with_progname = true;
 	    }
 	}	    
@@ -1087,14 +1110,18 @@ msgdomain_list_print (msgdomain_list_ty *mdlp, const char *filename,
 
   if (use_syntax_properties)
     msgdomain_list_print_properties (mdlp, fp, page_width, debug);
+  else if (use_syntax_stringtable)
+    msgdomain_list_print_stringtable (mdlp, fp, page_width, debug);
   else
     msgdomain_list_print_po (mdlp, fp, debug);
 
   /* Make sure nothing went wrong.  */
-  if (fflush (fp) || ferror (fp))
+  if (fwriteerror (fp))
     error (EXIT_FAILURE, errno, _("error while writing \"%s\" file"),
 	   filename);
-  fclose (fp);
+
+  if (fp != stdout)
+    fclose (fp);
 }
 
 
