@@ -1,5 +1,5 @@
 /* xgettext awk backend.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
@@ -28,11 +28,11 @@
 #include <string.h>
 
 #include "message.h"
-#include "x-awk.h"
 #include "xgettext.h"
+#include "x-awk.h"
 #include "error.h"
-#include "progname.h"
-#include "xmalloc.h"
+#include "error-progname.h"
+#include "xalloc.h"
 #include "exit.h"
 #include "gettext.h"
 
@@ -42,27 +42,6 @@
 /* The awk syntax is defined in the gawk manual page and documentation.
    See also gawk/awkgram.y.  */
 
-enum token_type_ty
-{
-  token_type_eof,
-  token_type_lparen,		/* ( */
-  token_type_rparen,		/* ) */
-  token_type_comma,		/* , */
-  token_type_string,		/* "abc" */
-  token_type_i18nstring,	/* _"abc" */
-  token_type_symbol,		/* symbol, number */
-  token_type_other		/* regexp, misc. operator */
-};
-typedef enum token_type_ty token_type_ty;
-
-typedef struct token_ty token_ty;
-struct token_ty
-{
-  token_type_ty type;
-  char *string;		/* for token_type_{symbol,string,i18nstring} */
-  int line_number;
-};
-
 
 /* ====================== Keyword set customization.  ====================== */
 
@@ -123,8 +102,17 @@ init_keywords ()
     }
 }
 
+void
+init_flag_table_awk ()
+{
+  xgettext_record_flag ("dcgettext:1:pass-awk-format");
+  xgettext_record_flag ("dcngettext:1:pass-awk-format");
+  xgettext_record_flag ("dcngettext:2:pass-awk-format");
+  xgettext_record_flag ("printf:1:awk-format");
+}
 
-/* ================== Reading of characters and tokens.  =================== */
+
+/* ======================== Reading of characters.  ======================== */
 
 /* Real filename, used in error messages about the input file.  */
 static const char *real_file_name;
@@ -199,16 +187,20 @@ phase2_getc ()
 	  c = phase1_getc ();
 	  if (c == '\n' || c == EOF)
 	    break;
-	  if (buflen >= bufmax)
+	  /* We skip all leading white space, but not EOLs.  */
+	  if (!(buflen == 0 && (c == ' ' || c == '\t')))
 	    {
-	      bufmax += 100;
-	      buffer = xrealloc (buffer, bufmax);
+	      if (buflen >= bufmax)
+		{
+		  bufmax = 2 * bufmax + 10;
+		  buffer = xrealloc (buffer, bufmax);
+		}
+	      buffer[buflen++] = c;
 	    }
-	  buffer[buflen++] = c;
 	}
       if (buflen >= bufmax)
 	{
-	  bufmax += 100;
+	  bufmax = 2 * bufmax + 10;
 	  buffer = xrealloc (buffer, bufmax);
 	}
       buffer[buflen] = '\0';
@@ -226,6 +218,32 @@ phase2_ungetc (int c)
 }
 
 
+/* ========================== Reading of tokens.  ========================== */
+
+
+enum token_type_ty
+{
+  token_type_eof,
+  token_type_lparen,		/* ( */
+  token_type_rparen,		/* ) */
+  token_type_comma,		/* , */
+  token_type_string,		/* "abc" */
+  token_type_i18nstring,	/* _"abc" */
+  token_type_symbol,		/* symbol, number */
+  token_type_semicolon,		/* ; */
+  token_type_other		/* regexp, misc. operator */
+};
+typedef enum token_type_ty token_type_ty;
+
+typedef struct token_ty token_ty;
+struct token_ty
+{
+  token_type_ty type;
+  char *string;		/* for token_type_{symbol,string,i18nstring} */
+  int line_number;
+};
+
+
 /* 7. Replace escape sequences within character strings with their
    single character equivalents.  */
 
@@ -382,7 +400,9 @@ x_awk_lex (token_ty *tp)
 	  if (last_non_comment_line > last_comment_line)
 	    xgettext_comment_reset ();
 	  /* Newline is not allowed inside expressions.  It usually
-	     introduces a fresh statement.  */
+	     introduces a fresh statement.
+	     FIXME: Newlines after any of ',' '{' '?' ':' '||' '&&' 'do' 'else'
+	     does *not* introduce a fresh statement.  */
 	  prefer_division_over_regexp = false;
 	  /* FALLTHROUGH */
 	case '\t':
@@ -431,7 +451,7 @@ x_awk_lex (token_ty *tp)
 	    {
 	      if (bufpos >= bufmax)
 		{
-		  bufmax += 100;
+		  bufmax = 2 * bufmax + 10;
 		  buffer = xrealloc (buffer, bufmax);
 		}
 	      buffer[bufpos++] = c;
@@ -465,7 +485,7 @@ x_awk_lex (token_ty *tp)
 	    }
 	  if (bufpos >= bufmax)
 	    {
-	      bufmax += 100;
+	      bufmax = 2 * bufmax + 10;
 	      buffer = xrealloc (buffer, bufmax);
 	    }
 	  buffer[bufpos] = '\0';
@@ -504,14 +524,14 @@ x_awk_lex (token_ty *tp)
 		break;
 	      if (bufpos >= bufmax)
 		{
-		  bufmax += 100;
+		  bufmax = 2 * bufmax + 10;
 		  buffer = xrealloc (buffer, bufmax);
 		}
 	      buffer[bufpos++] = c;
 	    }
 	  if (bufpos >= bufmax)
 	    {
-	      bufmax += 100;
+	      bufmax = 2 * bufmax + 10;
 	      buffer = xrealloc (buffer, bufmax);
 	    }
 	  buffer[bufpos] = '\0';
@@ -534,6 +554,11 @@ x_awk_lex (token_ty *tp)
 	  prefer_division_over_regexp = false;
 	  return;
 
+	case ';':
+	  tp->type = token_type_semicolon;
+	  prefer_division_over_regexp = false;
+	  return;
+
 	case ']':
 	  tp->type = token_type_other;
 	  prefer_division_over_regexp = true;
@@ -639,6 +664,11 @@ x_awk_lex (token_ty *tp)
 
 /* ========================= Extracting strings.  ========================== */
 
+
+/* Context lookup table.  */
+static flag_context_list_table_ty *flag_context_list_table;
+
+
 /* The file is broken into tokens.  Scan the token stream, looking for
    a keyword, followed by a left paren, followed by a string.  When we
    see this sequence, we have something to remember.  We assume we are
@@ -663,6 +693,8 @@ x_awk_lex (token_ty *tp)
    Return true upon eof, false upon closing parenthesis.  */
 static bool
 extract_parenthesized (message_list_ty *mlp,
+		       flag_context_ty outer_context,
+		       flag_context_list_iterator_ty context_iter,
 		       int commas_to_skip, int plural_commas)
 {
   /* Remember the message containing the msgid, for msgid_plural.  */
@@ -673,6 +705,16 @@ extract_parenthesized (message_list_ty *mlp,
   /* Parameters of the keyword just seen.  Defined only in state 1.  */
   int next_commas_to_skip = -1;
   int next_plural_commas = 0;
+  /* Whether to implicitly assume the next tokens are arguments even without
+     a '('.  */
+  bool next_is_argument = false;
+  /* Context iterator that will be used if the next token is a '('.  */
+  flag_context_list_iterator_ty next_context_iter =
+    passthrough_context_list_iterator;
+  /* Current context.  */
+  flag_context_ty inner_context =
+    inherited_context (outer_context,
+		       flag_context_list_iterator_advance (&context_iter));
 
   /* Start state is 0.  */
   state = 0;
@@ -682,53 +724,63 @@ extract_parenthesized (message_list_ty *mlp,
       token_ty token;
 
       x_awk_lex (&token);
+
+      if (next_is_argument && token.type != token_type_lparen)
+	{
+	  /* An argument list starts, even though there is no '('.  */
+	  context_iter = next_context_iter;
+	  outer_context = inner_context;
+	  inner_context =
+	    inherited_context (outer_context,
+			       flag_context_list_iterator_advance (
+				 &context_iter));
+	}
+
       switch (token.type)
 	{
 	case token_type_symbol:
-	  /* No need to bother if we extract all strings anyway.  */
-	  if (!extract_all)
-	    {
-	      void *keyword_value;
+	  {
+	    void *keyword_value;
 
-	      if (find_entry (&keywords, token.string, strlen (token.string),
-			      &keyword_value)
-		  == 0)
-		{
-		  int argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
-		  int argnum2 = (int) (long) keyword_value >> 10;
+	    if (find_entry (&keywords, token.string, strlen (token.string),
+			    &keyword_value)
+		== 0)
+	      {
+		int argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
+		int argnum2 = (int) (long) keyword_value >> 10;
 
-		  next_commas_to_skip = argnum1 - 1;
-		  next_plural_commas = (argnum2 > argnum1 ? argnum2 - argnum1 : 0);
-		  state = 1;
-		}
-	      else
-		state = 0;
-	    }
+		next_commas_to_skip = argnum1 - 1;
+		next_plural_commas = (argnum2 > argnum1 ? argnum2 - argnum1 : 0);
+		state = 1;
+	      }
+	    else
+	      state = 0;
+	  }
+	  next_is_argument =
+	    (strcmp (token.string, "print") == 0
+	     || strcmp (token.string, "printf") == 0);
+	  next_context_iter =
+	    flag_context_list_iterator (
+	      flag_context_list_table_lookup (
+		flag_context_list_table,
+		token.string, strlen (token.string)));
 	  free (token.string);
 	  continue;
 
 	case token_type_lparen:
-	  /* No need to recurse if we extract all strings anyway.  */
-	  if (extract_all)
-	    continue;
-	  if (state
-	      ?  extract_parenthesized (mlp, next_commas_to_skip,
-					next_plural_commas)
-	      : extract_parenthesized (mlp, -1, 0))
+	  if (extract_parenthesized (mlp, inner_context, next_context_iter,
+				     state ? next_commas_to_skip : -1,
+				     state ? next_plural_commas : 0))
 	    return true;
+	  next_is_argument = false;
+	  next_context_iter = null_context_list_iterator;
 	  state = 0;
 	  continue;
 
 	case token_type_rparen:
-	  /* No need to return if we extract all strings anyway.  */
-	  if (extract_all)
-	    continue;
 	  return false;
 
 	case token_type_comma:
-	  /* No need to bother if we extract all strings anyway.  */
-	  if (extract_all)
-	    continue;
 	  if (commas_to_skip >= 0)
 	    {
 	      if (commas_to_skip > 0)
@@ -742,6 +794,12 @@ extract_parenthesized (message_list_ty *mlp,
 		else
 		  commas_to_skip = -1;
 	    }
+	  inner_context =
+	    inherited_context (outer_context,
+			       flag_context_list_iterator_advance (
+				 &context_iter));
+	  next_is_argument = false;
+	  next_context_iter = passthrough_context_list_iterator;
 	  state = 0;
 	  continue;
 
@@ -752,7 +810,7 @@ extract_parenthesized (message_list_ty *mlp,
 	    pos.line_number = token.line_number;
 
 	    if (extract_all)
-	      remember_a_message (mlp, token.string, &pos);
+	      remember_a_message (mlp, token.string, inner_context, &pos);
 	    else
 	      {
 		if (commas_to_skip == 0)
@@ -760,8 +818,9 @@ extract_parenthesized (message_list_ty *mlp,
 		    if (plural_mp == NULL)
 		      {
 			/* Seen an msgid.  */
-			message_ty *mp = remember_a_message (mlp, token.string,
-							     &pos);
+			message_ty *mp =
+			  remember_a_message (mlp, token.string,
+					      inner_context, &pos);
 			if (plural_commas > 0)
 			  plural_mp = mp;
 		      }
@@ -769,16 +828,18 @@ extract_parenthesized (message_list_ty *mlp,
 		      {
 			/* Seen an msgid_plural.  */
 			remember_a_message_plural (plural_mp, token.string,
-						   &pos);
+						   inner_context, &pos);
 			plural_mp = NULL;
 		      }
 		  }
 		else
 		  free (token.string);
-		state = 0;
 	      }
-	    continue;
 	  }
+	  next_is_argument = false;
+	  next_context_iter = null_context_list_iterator;
+	  state = 0;
+	  continue;
 
 	case token_type_i18nstring:
 	  {
@@ -786,8 +847,28 @@ extract_parenthesized (message_list_ty *mlp,
 	    pos.file_name = logical_file_name;
 	    pos.line_number = token.line_number;
 
-	    remember_a_message (mlp, token.string, &pos);
+	    remember_a_message (mlp, token.string, inner_context, &pos);
 	  }
+	  next_is_argument = false;
+	  next_context_iter = null_context_list_iterator;
+	  state = 0;
+	  continue;
+
+	case token_type_semicolon:
+	  /* An argument list ends, and a new statement begins.  */
+	  /* FIXME: Should handle newline that acts as statement separator
+	     in the same way.  */
+	  /* FIXME: Instead of resetting outer_context here, it may be better
+	     to recurse in the next_is_argument handling above, waiting for
+	     the next semicolon or other statement terminator.  */
+	  outer_context = null_context;
+	  context_iter = null_context_list_iterator;
+	  next_is_argument = false;
+	  next_context_iter = passthrough_context_list_iterator;
+	  inner_context =
+	    inherited_context (outer_context,
+			       flag_context_list_iterator_advance (
+				 &context_iter));
 	  state = 0;
 	  continue;
 
@@ -795,6 +876,8 @@ extract_parenthesized (message_list_ty *mlp,
 	  return true;
 
 	case token_type_other:
+	  next_is_argument = false;
+	  next_context_iter = null_context_list_iterator;
 	  state = 0;
 	  continue;
 
@@ -808,6 +891,7 @@ extract_parenthesized (message_list_ty *mlp,
 void
 extract_awk (FILE *f,
 	     const char *real_filename, const char *logical_filename,
+	     flag_context_list_table_ty *flag_table,
 	     msgdomain_list_ty *mdlp)
 {
   message_list_ty *mlp = mdlp->item[0]->messages;
@@ -822,11 +906,14 @@ extract_awk (FILE *f,
 
   prefer_division_over_regexp = false;
 
+  flag_context_list_table = flag_table;
+
   init_keywords ();
 
   /* Eat tokens until eof is seen.  When extract_parenthesized returns
      due to an unbalanced closing parenthesis, just restart it.  */
-  while (!extract_parenthesized (mlp, -1, 0))
+  while (!extract_parenthesized (mlp, null_context, null_context_list_iterator,
+				 -1, 0))
     ;
 
   fp = NULL;
