@@ -29,15 +29,13 @@
 #include <string.h>
 
 #include "message.h"
-#include "x-python.h"
 #include "xgettext.h"
+#include "x-python.h"
 #include "error.h"
-#include "progname.h"
-#include "xmalloc.h"
+#include "error-progname.h"
+#include "xalloc.h"
 #include "exit.h"
 #include "po-charset.h"
-#include "msgl-ascii.h"
-#include "msgl-iconv.h"
 #include "uniname.h"
 #include "utf16-ucs4.h"
 #include "ucs4-utf8.h"
@@ -51,26 +49,6 @@
    See also Python-2.0/Parser/tokenizer.c, Python-2.0/Python/compile.c,
    Python-2.0/Objects/unicodeobject.c.  */
 
-enum token_type_ty
-{
-  token_type_eof,
-  token_type_lparen,		/* ( */
-  token_type_rparen,		/* ) */
-  token_type_comma,		/* , */
-  token_type_string,		/* "abc", 'abc', """abc""", '''abc''' */
-  token_type_symbol,		/* symbol, number */
-  token_type_other		/* misc. operator */
-};
-typedef enum token_type_ty token_type_ty;
-
-typedef struct token_ty token_ty;
-struct token_ty
-{
-  token_type_ty type;
-  char *string;		/* for token_type_string, token_type_symbol */
-  int line_number;
-};
-
 
 /* ====================== Keyword set customization.  ====================== */
 
@@ -136,8 +114,24 @@ init_keywords ()
     }
 }
 
+void
+init_flag_table_python ()
+{
+  xgettext_record_flag ("gettext:1:pass-python-format");
+  xgettext_record_flag ("ugettext:1:pass-python-format");
+  xgettext_record_flag ("dgettext:2:pass-python-format");
+  xgettext_record_flag ("ngettext:1:pass-python-format");
+  xgettext_record_flag ("ngettext:2:pass-python-format");
+  xgettext_record_flag ("ungettext:1:pass-python-format");
+  xgettext_record_flag ("ungettext:2:pass-python-format");
+  xgettext_record_flag ("dngettext:2:pass-python-format");
+  xgettext_record_flag ("dngettext:3:pass-python-format");
+  xgettext_record_flag ("_:1:pass-python-format");
+  /* xgettext_record_flag ("%:1:python-format"); // % is an infix operator! */
+}
 
-/* ================== Reading of characters and tokens.  =================== */
+
+/* ======================== Reading of characters.  ======================== */
 
 /* Real filename, used in error messages about the input file.  */
 static const char *real_file_name;
@@ -149,11 +143,6 @@ static int line_number;
 /* The input file stream.  */
 static FILE *fp;
 
-/* These are for tracking whether comments count as immediately before
-   keyword.  */
-static int last_comment_line;
-static int last_non_comment_line;
-
 
 /* 1. line_number handling.  Also allow a lookahead of 9 characters.  */
 
@@ -221,7 +210,7 @@ comment_add (int c)
   size_t len = ((unsigned char) c < 0x80 ? 1 : 2);
   if (buflen + len > bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   if ((unsigned char) c < 0x80)
@@ -241,13 +230,18 @@ comment_line_end ()
     --buflen;
   if (buflen >= bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
   xgettext_comment_add (buffer);
 }
 
+/* These are for tracking whether comments count as immediately before
+   keyword.  */
+static int last_comment_line;
+static int last_non_comment_line;
+
 
 /* 2. Outside strings, replace backslash-newline with nothing and a comment
       with nothing.  */
@@ -275,6 +269,7 @@ phase2_getc ()
       else if (c == '#')
 	{
 	  /* Eat a comment.  */
+	  last_comment_line = line_number;
 	  comment_start ();
 	  for (;;)
 	    {
@@ -300,6 +295,30 @@ phase2_ungetc (int c)
 }
 
 
+/* ========================== Reading of tokens.  ========================== */
+
+
+enum token_type_ty
+{
+  token_type_eof,
+  token_type_lparen,		/* ( */
+  token_type_rparen,		/* ) */
+  token_type_comma,		/* , */
+  token_type_string,		/* "abc", 'abc', """abc""", '''abc''' */
+  token_type_symbol,		/* symbol, number */
+  token_type_other		/* misc. operator */
+};
+typedef enum token_type_ty token_type_ty;
+
+typedef struct token_ty token_ty;
+struct token_ty
+{
+  token_type_ty type;
+  char *string;		/* for token_type_string, token_type_symbol */
+  int line_number;
+};
+
+
 /* There are two different input syntaxes for strings, "abc" and r"abc",
    and two different input syntaxes for Unicode strings, u"abc" and ur"abc".
    Which escape sequences are understood, i.e. what is interpreted specially
@@ -678,7 +697,12 @@ phase5_get (token_ty *tp)
 	    continue;
 	  tp->type = token_type_other;
 	  return;
+	}
 
+      last_non_comment_line = tp->line_number;
+
+      switch (c)
+	{
 	case '.':
 	  {
 	    int c1 = phase2_getc ();
@@ -716,7 +740,7 @@ phase5_get (token_ty *tp)
 	      {
 		if (bufpos >= bufmax)
 		  {
-		    bufmax += 100;
+		    bufmax = 2 * bufmax + 10;
 		    buffer = xrealloc (buffer, bufmax);
 		  }
 		buffer[bufpos++] = c;
@@ -745,7 +769,7 @@ phase5_get (token_ty *tp)
 	      }
 	    if (bufpos >= bufmax)
 	      {
-		bufmax += 100;
+		bufmax = 2 * bufmax + 10;
 		buffer = xrealloc (buffer, bufmax);
 	      }
 	    buffer[bufpos] = '\0';
@@ -847,7 +871,7 @@ phase5_get (token_ty *tp)
 		  len = (uc < 0x10000 ? 1 : 2);
 		  if (bufpos + len > bufmax)
 		    {
-		      bufmax += 100;
+		      bufmax = 2 * bufmax + 10;
 		      buffer =
 			xrealloc (buffer, bufmax * sizeof (unsigned short));
 		    }
@@ -960,6 +984,11 @@ x_python_lex (token_ty *tp)
 
 /* ========================= Extracting strings.  ========================== */
 
+
+/* Context lookup table.  */
+static flag_context_list_table_ty *flag_context_list_table;
+
+
 /* The file is broken into tokens.  Scan the token stream, looking for
    a keyword, followed by a left paren, followed by a string.  When we
    see this sequence, we have something to remember.  We assume we are
@@ -984,6 +1013,8 @@ x_python_lex (token_ty *tp)
    Return true upon eof, false upon closing parenthesis.  */
 static bool
 extract_parenthesized (message_list_ty *mlp,
+		       flag_context_ty outer_context,
+		       flag_context_list_iterator_ty context_iter,
 		       int commas_to_skip, int plural_commas)
 {
   /* Remember the message containing the msgid, for msgid_plural.  */
@@ -994,6 +1025,13 @@ extract_parenthesized (message_list_ty *mlp,
   /* Parameters of the keyword just seen.  Defined only in state 1.  */
   int next_commas_to_skip = -1;
   int next_plural_commas = 0;
+  /* Context iterator that will be used if the next token is a '('.  */
+  flag_context_list_iterator_ty next_context_iter =
+    passthrough_context_list_iterator;
+  /* Current context.  */
+  flag_context_ty inner_context =
+    inherited_context (outer_context,
+		       flag_context_list_iterator_advance (&context_iter));
 
   /* Start state is 0.  */
   state = 0;
@@ -1006,50 +1044,44 @@ extract_parenthesized (message_list_ty *mlp,
       switch (token.type)
 	{
 	case token_type_symbol:
-	  /* No need to bother if we extract all strings anyway.  */
-	  if (!extract_all)
-	    {
-	      void *keyword_value;
+	  {
+	    void *keyword_value;
 
-	      if (find_entry (&keywords, token.string, strlen (token.string),
-			      &keyword_value)
-		  == 0)
-		{
-		  int argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
-		  int argnum2 = (int) (long) keyword_value >> 10;
+	    if (find_entry (&keywords, token.string, strlen (token.string),
+			    &keyword_value)
+		== 0)
+	      {
+		int argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
+		int argnum2 = (int) (long) keyword_value >> 10;
 
-		  next_commas_to_skip = argnum1 - 1;
-		  next_plural_commas = (argnum2 > argnum1 ? argnum2 - argnum1 : 0);
-		  state = 1;
-		}
-	      else
-		state = 0;
-	    }
+		next_commas_to_skip = argnum1 - 1;
+		next_plural_commas = (argnum2 > argnum1 ? argnum2 - argnum1 : 0);
+		state = 1;
+	      }
+	    else
+	      state = 0;
+	  }
+	  next_context_iter =
+	    flag_context_list_iterator (
+	      flag_context_list_table_lookup (
+		flag_context_list_table,
+		token.string, strlen (token.string)));
 	  free (token.string);
 	  continue;
 
 	case token_type_lparen:
-	  /* No need to recurse if we extract all strings anyway.  */
-	  if (extract_all)
-	    continue;
-	  if (state
-	      ?  extract_parenthesized (mlp, next_commas_to_skip,
-					next_plural_commas)
-	      : extract_parenthesized (mlp, -1, 0))
+	  if (extract_parenthesized (mlp, inner_context, next_context_iter,
+				     state ? next_commas_to_skip : -1,
+				     state ? next_plural_commas : 0))
 	    return true;
+	  next_context_iter = null_context_list_iterator;
 	  state = 0;
 	  continue;
 
 	case token_type_rparen:
-	  /* No need to return if we extract all strings anyway.  */
-	  if (extract_all)
-	    continue;
 	  return false;
 
 	case token_type_comma:
-	  /* No need to bother if we extract all strings anyway.  */
-	  if (extract_all)
-	    continue;
 	  if (commas_to_skip >= 0)
 	    {
 	      if (commas_to_skip > 0)
@@ -1063,6 +1095,11 @@ extract_parenthesized (message_list_ty *mlp,
 		else
 		  commas_to_skip = -1;
 	    }
+	  inner_context =
+	    inherited_context (outer_context,
+			       flag_context_list_iterator_advance (
+				 &context_iter));
+	  next_context_iter = passthrough_context_list_iterator;
 	  state = 0;
 	  continue;
 
@@ -1073,7 +1110,7 @@ extract_parenthesized (message_list_ty *mlp,
 	    pos.line_number = token.line_number;
 
 	    if (extract_all)
-	      remember_a_message (mlp, token.string, &pos);
+	      remember_a_message (mlp, token.string, inner_context, &pos);
 	    else
 	      {
 		if (commas_to_skip == 0)
@@ -1081,8 +1118,9 @@ extract_parenthesized (message_list_ty *mlp,
 		    if (plural_mp == NULL)
 		      {
 			/* Seen an msgid.  */
-			message_ty *mp = remember_a_message (mlp, token.string,
-							     &pos);
+			message_ty *mp =
+			  remember_a_message (mlp, token.string,
+					      inner_context, &pos);
 			if (plural_commas > 0)
 			  plural_mp = mp;
 		      }
@@ -1090,21 +1128,23 @@ extract_parenthesized (message_list_ty *mlp,
 		      {
 			/* Seen an msgid_plural.  */
 			remember_a_message_plural (plural_mp, token.string,
-						   &pos);
+						   inner_context, &pos);
 			plural_mp = NULL;
 		      }
 		  }
 		else
 		  free (token.string);
-		state = 0;
 	      }
-	    continue;
 	  }
+	  next_context_iter = null_context_list_iterator;
+	  state = 0;
+	  continue;
 
 	case token_type_eof:
 	  return true;
 
 	case token_type_other:
+	  next_context_iter = null_context_list_iterator;
 	  state = 0;
 	  continue;
 
@@ -1118,6 +1158,7 @@ extract_parenthesized (message_list_ty *mlp,
 void
 extract_python (FILE *f,
 		const char *real_filename, const char *logical_filename,
+		flag_context_list_table_ty *flag_table,
 		msgdomain_list_ty *mdlp)
 {
   message_list_ty *mlp = mdlp->item[0]->messages;
@@ -1135,11 +1176,14 @@ extract_python (FILE *f,
 
   open_pbb = 0;
 
+  flag_context_list_table = flag_table;
+
   init_keywords ();
 
   /* Eat tokens until eof is seen.  When extract_parenthesized returns
      due to an unbalanced closing parenthesis, just restart it.  */
-  while (!extract_parenthesized (mlp, -1, 0))
+  while (!extract_parenthesized (mlp, null_context, null_context_list_iterator,
+				 -1, 0))
     ;
 
   fp = NULL;
