@@ -1,5 +1,5 @@
 /* xgettext RST backend.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
@@ -27,11 +27,11 @@
 #include <stddef.h>
 
 #include "message.h"
-#include "x-rst.h"
 #include "xgettext.h"
+#include "x-rst.h"
 #include "error.h"
-#include "progname.h"
-#include "xmalloc.h"
+#include "error-progname.h"
+#include "xalloc.h"
 #include "exit.h"
 #include "gettext.h"
 
@@ -57,6 +57,7 @@
 void
 extract_rst (FILE *f,
 	     const char *real_filename, const char *logical_filename,
+	     flag_context_list_table_ty *flag_table,
 	     msgdomain_list_ty *mdlp)
 {
   static char *buffer;
@@ -109,13 +110,15 @@ extract_rst (FILE *f,
 	    }
 	  if (bufpos >= bufmax)
 	    {
-	      bufmax += 100;
+	      bufmax = 2 * bufmax + 10;
 	      buffer = xrealloc (buffer, bufmax);
 	    }
 	  if (c == '=')
 	    break;
 	  buffer[bufpos++] = c;
 	  c = getc (f);
+	  if (c == EOF && ferror (f))
+	    goto bomb;
 	}
       buffer[bufpos] = '\0';
       location = xstrdup (buffer);
@@ -143,7 +146,7 @@ extract_rst (FILE *f,
 		    break;
 		  if (bufpos >= bufmax)
 		    {
-		      bufmax += 100;
+		      bufmax = 2 * bufmax + 10;
 		      buffer = xrealloc (buffer, bufmax);
 		    }
 		  buffer[bufpos++] = c;
@@ -160,6 +163,8 @@ extract_rst (FILE *f,
 	    {
 	      int n;
 	      c = getc (f);
+	      if (c == EOF && ferror (f))
+		goto bomb;
 	      if (c == EOF || !isdigit (c))
 		{
 		  error_with_progname = false;
@@ -177,7 +182,7 @@ extract_rst (FILE *f,
 		}
 	      if (bufpos >= bufmax)
 		{
-		  bufmax += 100;
+		  bufmax = 2 * bufmax + 10;
 		  buffer = xrealloc (buffer, bufmax);
 		}
 	      buffer[bufpos++] = (unsigned char) n;
@@ -205,7 +210,7 @@ extract_rst (FILE *f,
 	}
       if (bufpos >= bufmax)
 	{
-	  bufmax += 100;
+	  bufmax = 2 * bufmax + 10;
 	  buffer = xrealloc (buffer, bufmax);
 	}
       buffer[bufpos] = '\0';
@@ -214,7 +219,7 @@ extract_rst (FILE *f,
       pos.file_name = location;
       pos.line_number = (size_t)(-1);
 
-      remember_a_message (mlp, msgid, &pos);
+      remember_a_message (mlp, msgid, null_context, &pos);
 
       /* Here c is the last read character: EOF or '\n'.  */
       if (c == EOF)
@@ -222,6 +227,9 @@ extract_rst (FILE *f,
     }
 
   if (ferror (f))
-    error (EXIT_FAILURE, errno, _("error while reading \"%s\""),
-	   real_filename);
+    {
+    bomb:
+      error (EXIT_FAILURE, errno, _("error while reading \"%s\""),
+	     real_filename);
+    }
 }
