@@ -1,5 +1,5 @@
 /* xgettext Smalltalk backend.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
@@ -26,10 +26,10 @@
 #include <stdlib.h>
 
 #include "message.h"
-#include "x-smalltalk.h"
 #include "xgettext.h"
+#include "x-smalltalk.h"
 #include "error.h"
-#include "xmalloc.h"
+#include "xalloc.h"
 #include "exit.h"
 #include "gettext.h"
 
@@ -60,24 +60,6 @@
    In well-formed expressions, {} and [] and () are balanced.
  */
 
-enum token_type_ty
-{
-  token_type_eof,
-  token_type_uniq,		/* # */
-  token_type_symbol,		/* symbol */
-  token_type_string_literal,	/* string, stringconst, symbolconst */
-  token_type_other		/* misc. operator */
-};
-typedef enum token_type_ty token_type_ty;
-
-typedef struct token_ty token_ty;
-struct token_ty
-{
-  token_type_ty type;
-  char *string;		/* for token_type_string_literal, token_type_symbol */
-  int line_number;
-};
-
 
 /* ======================== Reading of characters.  ======================== */
 
@@ -144,7 +126,7 @@ comment_add (int c)
 {
   if (buflen >= bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen++] = c;
@@ -158,7 +140,7 @@ comment_line_end ()
     --buflen;
   if (buflen >= bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
@@ -172,6 +154,28 @@ static int last_comment_line;
 static int last_non_comment_line;
 
 
+/* ========================== Reading of tokens.  ========================== */
+
+
+enum token_type_ty
+{
+  token_type_eof,
+  token_type_uniq,		/* # */
+  token_type_symbol,		/* symbol */
+  token_type_string_literal,	/* string, stringconst, symbolconst */
+  token_type_other		/* misc. operator */
+};
+typedef enum token_type_ty token_type_ty;
+
+typedef struct token_ty token_ty;
+struct token_ty
+{
+  token_type_ty type;
+  char *string;		/* for token_type_string_literal, token_type_symbol */
+  int line_number;
+};
+
+
 /* 2. Combine characters into tokens.  Discard comments and whitespace.  */
 
 /* Maximum used guaranteed to be < 2.  */
@@ -267,14 +271,14 @@ phase2_get (token_ty *tp)
 		}
 	      if (bufpos >= bufmax)
 		{
-		  bufmax += 100;
+		  bufmax = 2 * bufmax + 10;
 		  buffer = xrealloc (buffer, bufmax);
 		}
 	      buffer[bufpos++] = c;
 	    }
 	  if (bufpos >= bufmax)
 	    {
-	      bufmax += 100;
+	      bufmax = 2 * bufmax + 10;
 	      buffer = xrealloc (buffer, bufmax);
 	    }
 	  buffer[bufpos] = 0;
@@ -351,7 +355,7 @@ phase2_get (token_ty *tp)
 	    {
 	      if (bufpos >= bufmax)
 		{
-		  bufmax += 100;
+		  bufmax = 2 * bufmax + 10;
 		  buffer = xrealloc (buffer, bufmax);
 		}
 	      buffer[bufpos++] = c;
@@ -374,7 +378,7 @@ phase2_get (token_ty *tp)
 		case ':':
 		  if (bufpos >= bufmax)
 		    {
-		      bufmax += 100;
+		      bufmax = 2 * bufmax + 10;
 		      buffer = xrealloc (buffer, bufmax);
 		    }
 		  buffer[bufpos++] = c;
@@ -405,7 +409,7 @@ phase2_get (token_ty *tp)
 	    }
 	  if (bufpos >= bufmax)
 	    {
-	      bufmax += 100;
+	      bufmax = 2 * bufmax + 10;
 	      buffer = xrealloc (buffer, bufmax);
 	    }
 	  buffer[bufpos] = '\0';
@@ -477,6 +481,7 @@ x_smalltalk_lex (token_ty *tp)
 void
 extract_smalltalk (FILE *f,
 		   const char *real_filename, const char *logical_filename,
+		   flag_context_list_table_ty *flag_table,
 		   msgdomain_list_ty *mdlp)
 {
   message_list_ty *mlp = mdlp->item[0]->messages;
@@ -528,7 +533,7 @@ extract_smalltalk (FILE *f,
 		lex_pos_ty pos;
 		pos.file_name = logical_file_name;
 		pos.line_number = token.line_number;
-		remember_a_message (mlp, token.string, &pos);
+		remember_a_message (mlp, token.string, null_context, &pos);
 		state = 0;
 		break;
 	      }
@@ -537,7 +542,8 @@ extract_smalltalk (FILE *f,
 		lex_pos_ty pos;
 		pos.file_name = logical_file_name;
 		pos.line_number = token.line_number;
-		plural_mp = remember_a_message (mlp, token.string, &pos);
+		plural_mp = remember_a_message (mlp, token.string,
+						null_context, &pos);
 		state = 4;
 		break;
 	      }
@@ -546,7 +552,8 @@ extract_smalltalk (FILE *f,
 		lex_pos_ty pos;
 		pos.file_name = logical_file_name;
 		pos.line_number = token.line_number;
-		remember_a_message_plural (plural_mp, token.string, &pos);
+		remember_a_message_plural (plural_mp, token.string,
+					   null_context, &pos);
 		state = 0;
 		break;
 	      }
